package de.narz.vollabsetzungbarcode.VollabsetzungBarcode;

import java.io.InputStream;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.Security;
import java.security.Signature;
import java.security.cert.Certificate;
import java.util.Base64;
import org.bouncycastle.jce.provider.BouncyCastleProvider;


public class VollabsetzungBarcodeMitFiveRXKeystore {
	
	
	public VollabsetzungBarcodeMitFiveRXKeystore() {
    	try {
    		Security.addProvider(new BouncyCastleProvider());
    		
    		// Schlüssel aus dem Beispiel Keystore von FiveRX 01.10 lesen
			String passwort = "123456";
			InputStream stream = getClass().getResourceAsStream("./cert_and_key.p12");
			KeyStore store = KeyStore.getInstance("PKCS12");
			store.load(stream, passwort.toCharArray());
			PrivateKey privateKey = (PrivateKey)store.getKey("selfsigned", passwort.toCharArray());
			Certificate cert = store.getCertificate("selfsigned");
			PublicKey publicKey = cert.getPublicKey();
			
			// Inhalt des Barcodes
			String barcodeInhalt;
			{
				// Beispiel eRezept-ID definieren
				String eRezeptID = "160.000.000.000.000.57";
				System.out.println("Beispiel eRezept-ID: "+eRezeptID);
				
				// Signatur auf der eRezept-ID erzeugen
				SecureRandom secureRandom = new SecureRandom();
				Signature signature = Signature.getInstance("SHA256WithRSA", "BC");
				signature.initSign(privateKey, secureRandom);	
				byte [] sigBytes = eRezeptID.getBytes("UTF-8");
				signature.update(sigBytes);
				
				// Inhalt des Barcodes erstellen
				byte[] base64 = Base64.getEncoder().encode(signature.sign());
				barcodeInhalt = "VA:"+eRezeptID+":"+new String(base64, "UTF-8");
			}
			
			// Inhalt des Barcodes ausgeben
			System.out.println("Inhalt des Barcodes: "+barcodeInhalt);
			
			{
				// Barcodeinhalt wieder zerlegen ...
				String[] komponenten = barcodeInhalt.split(":");
				String eRezeptID = komponenten[1];
				String signatur = komponenten[2];
				System.out.println("gelesene eRezept-ID: "+eRezeptID);
				
				// ... und Signatur prüfen
				Signature signature = Signature.getInstance("SHA256WithRSA", "BC");
				signature.initVerify(publicKey);
				byte[] sigBytes = eRezeptID.getBytes("UTF-8");
				signature.update(sigBytes);

				// Prüfergebnis ausgeben
				if(signature.verify(Base64.getDecoder().decode(signatur))) {
					System.out.println("Signatur OK");
				} else {
					System.out.println("Signatur fehlerhaft");
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}  
	}

	public static void main(String[] args) {
		new VollabsetzungBarcodeMitFiveRXKeystore();
	}
}
