﻿using System;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Security.Cryptography.Xml;
using System.Text;
using System.Xml;

namespace ConsoleApp1 {
    public static class Program {
        private static void Main() {

            { //encrypt with XML
                var publicKey = new X509Certificate2();
                publicKey.Import("publickeyOnly.cer");

                // Create a new XML document.
                XmlDocument xmlDoc = new XmlDocument {PreserveWhitespace = true};

                // Load an XML file into the XmlDocument object.
                xmlDoc.Load("eDispensierung.xml");

                // Sign the XML document. -> das passiert später mit dem öffentlichen Schlüssel des RZs
                EncryptXml(publicKey, xmlDoc);

                Console.WriteLine("XML encrypted.");

                // Save the document.
                xmlDoc.Save("eDispensierung_encrypted.xml");
            }
            { //decrypt with XML
                var privateKey = new X509Certificate2();
                privateKey.Import("cert_and_key.p12", "123456", X509KeyStorageFlags.DefaultKeySet);

                // Create a new XML document.
                XmlDocument xmlDoc = new XmlDocument {PreserveWhitespace = true};

                // Load an XML file into the XmlDocument object.
                xmlDoc.Load("eDispensierung_encrypted.xml");

                // Sign the XML document. 
                DecryptXml(privateKey, xmlDoc);

                Console.WriteLine("XML decrypted.");

                // Save the document.
                xmlDoc.Save("eDispensierung_encrypted_decrypted.xml");
            }

            //Barcode für Vollabsetzung/Korrektur erzeugen und prüfen
            {
                string barcodeInhalt;
                {
                    var privateKey = new X509Certificate2();
                    privateKey.Import("cert_and_key.p12", "123456", X509KeyStorageFlags.DefaultKeySet);

                    // Beispiel eRezept-ID definieren
                    string eRezeptID = "160.000.000.000.000.57";
                    Console.Out.WriteLine("Beispiel eRezept-ID: " + eRezeptID);

                    // Signatur auf der eRezept-ID erzeugen
                    byte[] signature = privateKey.GetRSAPrivateKey().SignData(Encoding.UTF8.GetBytes(eRezeptID), HashAlgorithmName.SHA256, RSASignaturePadding.Pkcs1);

                    // Inhalt des Barcodes erstellen
                    barcodeInhalt = $"VA:{eRezeptID}:{Convert.ToBase64String(signature)}";

                    // Inhalt des Barcodes ausgeben
                    Console.Out.WriteLine("Inhalt des Barcodes: "+barcodeInhalt);
                }

                {
                    // Barcodeinhalt wieder zerlegen ...
                    string[] komponenten = barcodeInhalt.Split(':');
                    string eRezeptID = komponenten[1];
                    string signatur = komponenten[2];
                    Console.Out.WriteLine("gelesene eRezept-ID: "+eRezeptID);
				
                    // ... und Signatur prüfen mit PublicKey
                    var publicKey = new X509Certificate2();
                    publicKey.Import("publickeyOnly.cer");

                    var signature = publicKey.GetRSAPublicKey().VerifyData(
                        Encoding.UTF8.GetBytes(eRezeptID), 
                        Convert.FromBase64String(signatur), HashAlgorithmName.SHA256, RSASignaturePadding.Pkcs1);

                    // Prüfergebnis ausgeben
                    if(signature) {
                        Console.Out.WriteLine("Signatur OK");
                    } else {
                        Console.Out.WriteLine("Signatur fehlerhaft");
                    }
                }
            }
        }

        private static void EncryptXml(X509Certificate2 publicKey, XmlDocument doc) {
            XmlElement inputElement = doc.DocumentElement;

            RijndaelManaged sessionKey = new RijndaelManaged {KeySize = 256};
            EncryptedXml eXml = new EncryptedXml();
            byte[] encryptedElement = eXml.EncryptData(inputElement, sessionKey, true);

            EncryptedData edElement = new EncryptedData {
                Type = EncryptedXml.XmlEncElementContentUrl,
                EncryptionMethod = new EncryptionMethod(EncryptedXml.XmlEncAES256Url),
                CipherData = {CipherValue = encryptedElement}
            };

            byte[] encryptedKey = EncryptedXml.EncryptKey(sessionKey.Key, publicKey.GetRSAPublicKey(), true);

            EncryptedKey key = new EncryptedKey {
                CipherData = new CipherData(encryptedKey), EncryptionMethod = new EncryptionMethod(EncryptedXml.XmlEncRSAOAEPUrl)
            };

            edElement.KeyInfo = new KeyInfo();
            edElement.KeyInfo.AddClause(new KeyInfoEncryptedKey(key));

            EncryptedXml.ReplaceElement(inputElement, edElement, true);
        }

        private static void DecryptXml(X509Certificate2 privateKey, XmlDocument xmlDoc) {
            XmlElement encryptedElement = xmlDoc.GetElementsByTagName("EncryptedData")[0] as XmlElement;
            EncryptedData edElement = new EncryptedData();
            edElement.LoadXml(encryptedElement);

            foreach (var x in edElement.KeyInfo) {
                var z = x as KeyInfoEncryptedKey;

                var aeskey = EncryptedXml.DecryptKey(z.EncryptedKey.CipherData.CipherValue, privateKey.GetRSAPrivateKey(), true);

                RijndaelManaged sessionKey = new RijndaelManaged {KeySize = 256, Key = aeskey};

                EncryptedXml exml = new EncryptedXml(xmlDoc);
                byte[] rgbOutput = exml.DecryptData(edElement, sessionKey);
                exml.ReplaceData(encryptedElement, rgbOutput);

                break;
            }
        }
    }
}