# POC: XML verschlüsseln für die eDispensierung

#### Voraussetzungen

Voraussetzung ist, dass JDK8 oder neuer installiert ist. 

Mit Veresion 8 muss bei bestimmt Algorithmen und/oder Schlüssellängen die Java Cryptography Extension installiert sein.
Das erkennt man an Fehlern wie z.B. `java.security.InvalidKeyException: Illegal key size`.

https://dzone.com/articles/install-java-cryptography-extension-jce-unlimited

Neuere Versionen kommen bereits mit der passenden Erweiterung.

## Ausführung
Der POC ist grundsätzlich als Java-Projekt mit JUnit Tests ausgelegt. Die Schlüssel liegen als PEM Files vor bzw. kommen
aus einem Keystore, geliefert von Pharmatechnik.

Hinter einem Proxy kann es nötig sein, diesen zu konfigurieren. Dazu kann man eine Datei `gradle.properties` in das
Projektverzeichnis legen, mit folgendem Inhalt:

```
systemProp.http.proxyHost=theProxy
systemProp.http.proxyPort=8118
systemProp.https.proxyHost=theProxy
systemProp.https.proxyPort=8118 
```
#### Build

1. Projekt klonen / entpacken
1. In das Projektverzeichnis wechseln
1. `./gradlew build` oder `gradlew.bat build` aufrufen
1. Wenn man ein vollständiges, ausführbares JAR mit allen Abhängigkeiten erzeugen möchte, `./gradlew shadowJar` oder `gradlew.bat shadowJar` aufrufen 

#### Ergebnisse
Testergebnisse und Logausgaben können über einen Browser eingesehen werden: `build/reports/tests/test/index.html`

Ausgeführt wurden folgende Tests:

##### `TestXmlEncryptionHelper`
Ein Testcase, der eine XML Datei einliest, mittels XML Security verschlüsselt (nach Vorgabe RSA-OAEP und AES256-CBC, 
nur Java auch mit AES-GCM getestet) und danach wieder entschlüsselt.
Alle Dateien (original XML, verschlüsseltes XML und entschlüsseltes XML) werden nach `build/out/xml` geschrieben.

Dazu kommt ein Testcase, der ein mit .NET von Pharmatechnik generiertes, verschlüsseltes XML entschlüsselt und die Signatur validiert.

Der Testcase von Pharmatechnik verwendet den gleichen Keystore für Verschlüsselung und Signierung.

##### `TestHardcodedEnryptionHelper`
Ein Testcase, der eine XML Datei einliest, verschlüsselt und
Base64 enkodiert ausgibt. Danach werden diese enkodierten Daten wieder entschlüsselt.

Nach Vorgaben wird der symmetrische Schlüssel und ein IV generiert, mit den verschlüsselten Daten verknüpft.
 
Alle Dateien (original XML, Base64 enkodierte Daten entschlüsseltes XML) werden nach `build/out/base64` geschrieben.

Außerdem gibt es einen Test, der einen von Pharmatechnik generiertes Base64 enkodiertes und verschlüsseltes XML mit gelieferten Schlüsseln entschlüsselt.

## Kommandozeilentool (nur hardcoded)
Die Usage erhält man mit
```bash
> java -jar xmlsecpoc-all.jar --help

usage: java -jar xmlsecpoc-all.jar [encrypt|decrypt]
 -a,--alias <alias>         the alias of the key within the keystore
 -h,--help                  print usage
 -i,--in <infile>           the input file to read from
 -k,--key <keyfile>         the pkcs12 keystore file containing the key
 -o,--out <outfile>         the output file to write to
 -p,--password <password>   the password to access the keystore and the
                            private key
```

Beispiele:
```
> java -jar xmlsecpoc-all.jar encrypt -k cert_and_key.p12 -p 123456 -a selfsigned -i bootstrap.xml -o base64.txt
```
```
> java -jar xmlsecpoc-all.jar decrypt -k cert_and_key.p12 -p 123456 -a selfsigned -i base64.txt -o decrypted.xml
```

Getestet mit JRE8 + Crypto Extension und mit JRE11 auf Mac.

Beispiel für das Erstellen eine Schlüsselpaares mit `keytool`:
```
keytool -keystore signing_cert_and_key.p12 -storetype PKCS12 -genkey -alias serverkeys -keyalg RSA -keysize 4096 -validity 3650  -storepass 123456 -dname "CN=localhost, OU=myOU, O=myO, L=myL, S=myS, C=DE" -keypass 123456
```
