package de.noventi.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;

/**
 * Class abstract.
 * <p/>
 * Class description
 * <p/>
 *
 * @author zeitler
 */
public class DomUtil {

  /**
   * Logger
   */
  public static final Logger LOG = LoggerFactory.getLogger(DomUtil.class);

  public static void docToFile(Document document, String fileName) throws Exception {
    File file = new File(fileName);
    try (FileOutputStream f = new FileOutputStream(file)) {
      TransformerFactory factory = TransformerFactory.newInstance();
      Transformer transformer = factory.newTransformer();
      transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
      transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
      // transformer.setOutputProperty(OutputKeys.INDENT, "yes");

      if (document.getDoctype() != null) {
        String systemValue = (new File (document.getDoctype().getSystemId())).getName();
        transformer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM, systemValue);
      }

      DOMSource source = new DOMSource(document);
      StreamResult result = new StreamResult(f);
      transformer.transform(source, result);
    }

    LOG.info("outputDocToFile: document written! xmlFile={}", file.toURI().toURL().toString());
  }

  public static Document readXmlFromClasspath(String resource, boolean nameSpaceAware) throws Exception {
    InputStream is = Thread.currentThread().getContextClassLoader().getResourceAsStream(resource);
    Document doc = readXmlFromStream(is, nameSpaceAware);

    LOG.info("readXmlFromClasspath: resource read! resource={}", resource);

    return doc;
  }

  public static Document readXmlFromFile(File file, boolean nameSpaceAware) throws Exception {
    Document doc = readXmlFromStream(new FileInputStream(file), nameSpaceAware);

    LOG.info("readXmlFromFile: resource read! file={}", file);

    return doc;
  }

  private static Document readXmlFromStream(InputStream inputStream, boolean nameSpaceAware) throws Exception {
    try(InputStream is = inputStream) {
      DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
      dbFactory.setNamespaceAware(nameSpaceAware);
      DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();

      return dBuilder.parse(is);
    }

  }

}
