package de.noventi.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.nio.charset.Charset;
import java.nio.file.Files;

/**
 * Class abstract.
 * <p/>
 * Class description
 * <p/>
 *
 * @author zeitler
 */
public class FileUtil {

  public static String readFileWithEncoding(File file, String encoding) throws IOException {
    // return Files.readString(file.toPath(), Charset.forName(encoding));
    byte[] bytes = Files.readAllBytes(file.toPath());
    return new String(bytes, Charset.forName(encoding));
  }

  public static byte[] readFileAsBytes(File file) throws Exception {
    return Files.readAllBytes(file.toPath());
  }

  public static void writeDataWithEncoding(File file, byte[] data, Charset charset) throws IOException {
    String text = new String(data, charset);
    writeString(file, text);
  }

  public static void writeString(File file, String text) throws IOException {
    // Files.writeString(file.toPath(), text);
    try (Writer writer = new BufferedWriter(new FileWriter(file))) {
      writer.write(text);
    }
  }

  public static File getFileForReading(String fileName) {
    File file = new File(fileName);
    if (!file.exists()) {
      throw new IllegalArgumentException("File does not exist! fileName" + fileName);
    }
    if (!file.isFile()) {
      throw new IllegalArgumentException("File is not a file! fileName" + fileName);
    }
    if (!file.canRead()) {
      throw new IllegalArgumentException("Can not read file! fileName" + fileName);
    }
    return file;
  }

}
