package de.noventi.xmlsecpoc;

import static org.assertj.core.api.Assertions.assertThat;

import de.noventi.util.CryptoUtil;
import de.noventi.util.FileUtil;
import java.io.File;
import java.nio.charset.Charset;
import java.security.KeyPair;
import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Class abstract.
 * <p/>
 * Class description
 * <p/>
 *
 * @author zeitler
 */
public class TestHardcodedEncryptionHelper {

  /**
   * Logger
   */
  private static final Logger LOG = LoggerFactory.getLogger(TestHardcodedEncryptionHelper.class);

  private static final File outputDir = new File("./build/out/base64");

  private static final File resourceDir = new File("./src/test/resources");

  private static final String ENCODING = "UTF-8";

  @BeforeClass
  public static void beforeClass() {
    CryptoUtil.init();
    outputDir.mkdirs();
  }

  @Test
  public void testEncryptionDecryption() throws Exception {
    KeyPair encryptionKeyPair = retrieveKeyPair();

    File xmlFile = new File(resourceDir, "bootstrap.xml");
    String xmlContentAsString = FileUtil.readFileWithEncoding(xmlFile, ENCODING);

    LOG.info("testEncryptionDecryption: data={}{}", System.lineSeparator(), xmlContentAsString);

    byte[] xmlData = FileUtil.readFileAsBytes(xmlFile);
    FileUtil.writeDataWithEncoding(new File(outputDir, "original.xml"), xmlData, Charset.forName(ENCODING));

    byte[] byteWurst = HardcodedEncryptionHelper.encryptData(encryptionKeyPair.getPublic(), xmlData);
    String base64EncodedData = CryptoUtil.encodeBase64ToString(byteWurst);
    FileUtil.writeString(new File(outputDir, "base64.txt"), base64EncodedData);

    LOG.info("testEncryptionDecryption: base64 encoded result! result={}", base64EncodedData);

    byte[] decodedByteWurst = CryptoUtil.decodeBase64(base64EncodedData);
    byte[] decryptedData = HardcodedEncryptionHelper.decryptData(encryptionKeyPair.getPrivate(), decodedByteWurst);

    assertThat(decryptedData).containsExactly(xmlData);

    String xmlResult = new String(decryptedData, Charset.forName(ENCODING));
    LOG.info("testEncryptionDecryption: data={}{}", System.lineSeparator(), xmlResult);
    FileUtil.writeDataWithEncoding(new File(outputDir, "decrypted.xml"), decryptedData, Charset.forName(ENCODING));
  }

  @Test
  public void testProvidedBase64Data() throws Exception {
    File base64EncodedDataFile = new File(resourceDir, "pharmatechnik/hardcoded/output.txt");
    String base64EncodedData = FileUtil.readFileWithEncoding(base64EncodedDataFile, "UTF-8");
    byte[] encryptedData = CryptoUtil.decodeBase64(base64EncodedData);

    KeyPair keyPair = retrieveKeyPair();

    byte[] decryptData = HardcodedEncryptionHelper.decryptData(keyPair.getPrivate(), encryptedData);
    String xml = new String(decryptData, Charset.forName("UTF-8"));
    System.out.println(xml);
  }

  private KeyPair retrieveKeyPair() {
    File encrpytionKeyStore = new File(resourceDir, "pharmatechnik/hardcoded/encryption_cert_and_key.p12");
    KeyPair encryptionKeyPair = CryptoUtil.getKeyPairFromKeystore(encrpytionKeyStore, "123456".toCharArray(), "selfsigned");
    return encryptionKeyPair;
  }

}
